CASTING_BAR_ALPHA_STEP = 0.05;
CASTING_BAR_FLASH_STEP = 0.2;
CASTING_BAR_HOLD_TIME = 1;

MAX_PLAYER_BUFFS = 16;
MAX_PLAYER_DEBUFFS = 16;
MAX_PLAYER_BUFFS_SIZE = 20;

BUFF_FLASH_TIME = 2;
BUFF_ALPHA_VALUE = 1;
BUFF_MAX_ALPHA = 1.2;
BUFF_MIN_ALPHA = 0.2;
BUFF_DURATION_WARNING_TIME = 30;

BUFF_FONT_COLOR = {r = 0.0, g = 1.0, b = 0.0};
DEBUFF_FONT_COLOR = {r = 1.0, g = 1.0, b = 0.0};

function SUI_PlayerFrame_OnLoad(this)
	SUI_UnitFrame_Initialize(this, "player", 
	SUI_PlayerFrameName, SUI_PlayerFramePortrait, 
	SUI_PlayerFrameHealthBar, SUI_PlayerFrameManaBar, 
	SUI_PlayerFrameSkillBar, SUI_PlayerRaidTargetFrame);

	this:RegisterForClicks("LeftButton", "RightButton");
	this:RegisterEvent("LOADING_END");
	this:RegisterEvent("LOOT_METHOD_CHANGED");
	this:RegisterEvent("PARTY_LEADER_CHANGED");	
	this:RegisterEvent("UPDATE_LOOT_ASSIGN");
	this:RegisterEvent("UNIT_BUFF_CHANGED");
	
	SUI_PlayerFrame_Update(this);
end

function SUI_PlayerFrame_Update(this)	
	local damageValue = UnitChangeHealth("player");
	if (damageValue < 0) then
		this.showTimer = 0.5;
		SUI_PlayerHurtBorder:Show();
	end
	SUI_PlayerFrame_UpdateLeader(this);
	SUI_PlayerFrame_UpdateAssigner(this);
	SUI_PlayerFrame_UpdatePVPState(this);
end

function SUI_PlayerFrame_OnEvent(this, event, arg1, arg2)
	SUI_UnitFrame_OnEvent(this, event, arg1, arg2);
	if (event == "LOADING_END") then
		SUI_PlayerFrameName:SetText(UnitName("player"));
		SUI_PlayerFrameHealthBarInfoText:SetText(UnitRace("player"));
	elseif (event == "PARTY_LEADER_CHANGED") then
		SUI_PlayerFrame_UpdateLeader(this);
	elseif (event == "UPDATE_LOOT_ASSIGN" or event == "LOOT_METHOD_CHANGED") then
		SUI_PlayerFrame_UpdateAssigner(this);
	end
end

function SUI_PlayerFrame_OnClick(this, key)
	if (SpellIsTargeting() and key == "RBUTTON") then
		SpellStopCasting();
		return;
	end
	if (key == "LBUTTON") then
		if (SpellIsTargeting()) then
			SpellTargetUnit("player");
		else
			TargetUnit("player");
		end
	elseif (key == "RBUTTON") then
		ToggleDropDownMenu(PlayerFrameDropDown, 1, nil, "SUI_PlayerFrame", -2, 0);
	end
end

function SUI_PlayerFrame_UpdateLeader(this)
	if (IsPartyLeader()) then
		SUI_PlayerLeaderIcon:Show();
	else
		SUI_PlayerLeaderIcon:Hide();
	end
end

function SUI_PlayerFrame_UpdateAssigner(this)
	if (IsAssigner()) then
		SUI_PlayerAssignerButton:Show();
	else
		SUI_PlayerAssignerButton:Hide();
	end
end

function SUI_PlayerFrame_UpdatePVPState(this)
	if (UnitPKState("player")) then
		SUI_PlayerPVPStateIcon:Show();
	else
		SUI_PlayerPVPStateIcon:Hide();
	end
end

function SUI_PlayerAssignerButton_OnUpdate(this, elapsedTime)
	if (this.alphaValue == nil) then
		this.alphaValue = 0;
	end

	if (this.state) then
		this.alphaValue = this.alphaValue - elapsedTime;
		if (this.alphaValue < 0.2) then
			this.alphaValue = 0.2;
			this.state = nil;
		end
	else
		this.alphaValue = this.alphaValue + elapsedTime;
		if ( this.alphaValue > 1.2 ) then
			this.alphaValue = 1.2;
			this.state = 1;
		end
	end
	SUI_PlayerAssignerHighlightFrame:SetAlpha(this.alphaValue);
end

function SUI_PlayerAssignerButton_OnClick(this)
	if (GetLootAssignItemSize() == 0) then
		SendWarningMsg(DONT_OPEN_LOOTFRAME);
		return;
	end
	ShowUIPanel(LootFrame);
end

function SUI_PlayerFrame_OnUpdate(this, elapsedTime)
	if (this.showTimer) then
		if (this.showTimer < 0) then
			this.showTimer = nil;
			SUI_PlayerHurtBorder:Hide();
		else
			this.showTimer = this.showTimer - elapsedTime;
		end
	end

	SUI_PlayerFrame_UpdatePVPState(this);
end

function SUI_PlayerCastingBar_OnLoad(this)
	this:RegisterEvent("CASTING_START");
	this:RegisterEvent("CASTING_STOP");
	this:RegisterEvent("CASTING_FAILED");
	this:RegisterEvent("CASTING_DELAYED");
end

function SUI_PlayerCastingBar_OnEvent(this, event)
	if (event == "CASTING_START") then
		SUI_PlayerCastingBarText:SetText(arg1);
		SUI_PlayerCastingBarSpark:Show();
		SUI_PlayerCastingBarFlash:Hide();
		this.minValue = GetTime();
		this.maxValue = this.minValue + arg2;
		this.casting = 1;
		this.holdTime = 0;
		
		this:SetAlpha(1.0);
		this:SetBarColor(0.1, 0.7, 0.7);
		this:SetMinValue(this.minValue);
		this:SetMaxValue(this.maxValue);
		this:SetValue(this.minValue);
		this:Show();
	elseif (event == "CASTING_STOP") then
		if (this:IsVisible()) then
			SUI_PlayerCastingBarSpark:Hide();
			SUI_PlayerCastingBarFlash:SetAlpha(0.0);
			SUI_PlayerCastingBarFlash:Show();
			this:SetBarColor(0.0, 1.0, 0.0);
			this:SetValue(this.maxValue);
			this.casting = nil;
			this.flash = 1;
			this.fadeOut = 1;
		end
	elseif (event == "CASTING_FAILED") then
		if (this:IsVisible()) then
			SUI_PlayerCastingBarSpark:Hide();
			this:SetBarColor(1.0, 0.0, 0.0);
			this:SetValue(this.maxValue);
			this.casting = nil;
			this.fadeOut = 1;
			this.holdTime = GetTime() + CASTING_BAR_HOLD_TIME;
		end
	elseif (event == "CASTING_DELAYED") then
		if (this:IsVisible()) then
			this.minValue = this.minValue + arg1;
			this.maxValue = this.maxValue + arg1;
			this:SetMinValue(this.minValue);
			this:SetMaxValue(this.maxValue);
		end
	end
	SUI_PlayerCastingBar_SetAnchor();
end

function SUI_PlayerCastingBar_OnUpdate(this)
	if (this.casting) then
		local currTime = GetTime();
		if (currTime > this.maxValue) then
			currTime = this.maxValue;
		end
		
		this:SetValue(currTime);
		local sparkPosition = (currTime - this.minValue) / (this.maxValue - this.minValue) * 156;
		if (sparkPosition < 0) then
			sparkPosition = 0;
		end
		SUI_PlayerCastingBarSpark:ClearAllAnchors();
		SUI_PlayerCastingBarSpark:SetAnchor("CENTER", "LEFT", SUI_PlayerCastingBar, sparkPosition, 0);
	elseif (this.holdTime > GetTime()) then
		return;
	elseif ( this.flash ) then
		local alpha = SUI_PlayerCastingBarFlash:GetAlpha() + CASTING_BAR_FLASH_STEP;
		if (alpha < 1) then
			SUI_PlayerCastingBarFlash:SetAlpha(alpha);
		else
			this.flash = nil;
			SUI_PlayerCastingBarFlash:SetAlpha(1.0);
		end
	elseif (this.fadeOut) then
		local alpha = this:GetAlpha() - CASTING_BAR_ALPHA_STEP;
		if (alpha > 0) then
			this:SetAlpha(alpha);
		else
			this.fadeOut = nil;
			this:Hide();
		end
	end
end

function SUI_PlayerCastingBar_SetAnchor()
	local y = 3;
	local i = 1;
	while UnitBuff("player", i) ~= nil do
		if (i == 1) then
			y = y + MAX_PLAYER_BUFFS_SIZE;
		elseif (i == 9) then
			y = y + MAX_PLAYER_BUFFS_SIZE;
		end
		i = i + 1;
	end
	local i = 1;
	while UnitDebuff("player", i) ~= nil do
		if (i == 1) then
			y = y + MAX_PLAYER_BUFFS_SIZE;
		elseif (i == 9) then
			y = y + MAX_PLAYER_BUFFS_SIZE;
		end
		i = i + 1;
	end
	SUI_PlayerCastingBar:ClearAllAnchors();
	SUI_PlayerCastingBar:SetAnchor("TOP", "BOTTOM", "SUI_PlayerFrame", 0, y);
end

function SUI_PlayerBuffButton_Update(this)
	local name, texture, count;
	if (this.debuff) then
		name, texture, count = UnitDebuff("player", this:GetID());
		if this:GetName() == "SUI_PlayerDebuffButton1" then
			local i = 1;
			local y = 1;
			while UnitBuff("player",i) ~= nil do
				i = i + 1;
			end
			if i > 8 then
				y = 40;
			elseif i > 1 then
				y = 20;
			end
			this:ClearAllAnchors();
			this:SetAnchor("TOPLEFT", "BOTTOMLEFT", "SUI_PlayerBuffFrame", 0, y);
		end
	else
		name, texture, count = UnitBuff("player", this:GetID());
	end
	
	if (not name) then
		this:Hide();
	else
		if (count < 1) then
			count = "";
		end
		getglobal(this:GetName().."Icon"):SetTexture(texture);
		getglobal(this:GetName().."Count"):SetText(count);
		this:SetAlpha(1.0);
		this:Show();
	end
	
	if (GameTooltip:IsOwned(this)) then
		if (this.debuff) then
			GameTooltip:SetUnitDebuff("player", this:GetID());
		else
			GameTooltip:SetUnitBuff("player", this:GetID());
		end
	end
	
	SUI_PlayerBuffButton_OnUpdate(this);
end

function SUI_PlayerBuffButton_OnLoad(this)
	this:RegisterForClicks("RightButton");
	this:RegisterEvent("UNIT_BUFF_CHANGED");
	
	SUI_PlayerBuffButton_Update(this);
end

function SUI_PlayerBuffButton_OnEvent(this, event)
	if (event == "UNIT_BUFF_CHANGED") then
		if (arg1 == "player") then
			SUI_PlayerBuffButton_Update(this);
		end
	end
end

function SUI_PlayerBuffButton_OnUpdate(this, elapsedTime)
	local leftTime;
	if (this.debuff) then
		leftTime = UnitDebuffLeftTime("player", this:GetID());
	else
		leftTime = UnitBuffLeftTime("player", this:GetID());
		if (leftTime and leftTime < BUFF_DURATION_WARNING_TIME) then
			this:SetAlpha(BUFF_ALPHA_VALUE);
		else
			this:SetAlpha(1.0);
		end
	end
	SUI_PlayerFrame_UpdateDuration(this, leftTime);
end

function SUI_PlayerFrame_UpdateDuration(this, leftTime)
	local duration = getglobal(this:GetName().."Duration");
	if (leftTime) then
		duration:SetText(SUI_SecondsToTime(leftTime));
		if (this.debuff) then
			duration:SetColor(DEBUFF_FONT_COLOR.r, DEBUFF_FONT_COLOR.g, DEBUFF_FONT_COLOR.b);
		else
			duration:SetColor(BUFF_FONT_COLOR.r, BUFF_FONT_COLOR.g, BUFF_FONT_COLOR.b);
		end
		duration:Show();
	else
		duration:Hide();
	end
end

function SUI_PlayerBuffFrame_OnLoad(this)
	BUFF_ALPHA_VALUE = 1;
	BUFF_FLASH_STATE = 1;
end

function SUI_PlayerBuffFrame_OnUpdate(this, elapsedTime)
	if ( BUFF_FLASH_STATE ) then
		BUFF_ALPHA_VALUE = BUFF_ALPHA_VALUE - elapsedTime * BUFF_FLASH_TIME;
		if ( BUFF_ALPHA_VALUE < BUFF_MIN_ALPHA ) then
			BUFF_ALPHA_VALUE = BUFF_MIN_ALPHA;
			BUFF_FLASH_STATE = nil;
		end
	else
		BUFF_ALPHA_VALUE = BUFF_ALPHA_VALUE + elapsedTime * BUFF_FLASH_TIME;
		if ( BUFF_ALPHA_VALUE > BUFF_MAX_ALPHA ) then
			BUFF_ALPHA_VALUE = BUFF_MAX_ALPHA;
			BUFF_FLASH_STATE = 1;
		end
	end
end

function SUI_SecondsToTime(seconds)
	local getTime, unit;
	if ( seconds >= 86400  ) then
		getTime = math.ceil(seconds / 86400);
		return string.format("%d%s", getTime, getglobal("SUI_DAY"));
	end
	if ( seconds >= 3600  ) then
		getTime = math.ceil(seconds / 3600);
		return string.format("%d%s", getTime, getglobal("SUI_HOUR"));
	end
	if ( seconds >= 60  ) then
		getTime = math.ceil(seconds / 60);
		return string.format("%d%s", getTime, getglobal("SUI_MINUTE"));
	end
	return string.format("%d%s", seconds, getglobal("SUI_SECOND")); -- %.1f
end